//+------------------------------------------------------------------+
//|                                              LimitOrders (1).mq4 |
//|                                                Tatiana Zyrianova |
//|                        https://www.mql5.com/ru/users/mqldiligent |
//+------------------------------------------------------------------+
#property copyright "Tatiana Zyrianova"
#property link      "https://www.mql5.com/ru/users/mqldiligent"
#property version   "1.00"
#property strict

extern double       Lots           = 0.01;
extern int          Risk           = 3;
extern int          Indent         = 10;
extern int          DistanceOrders = 10;
extern int          CountOrders    = 3;
extern int          TrailingStop   = 10;
extern int          TrailingStep   = 1;
extern int          TakeProfit     = 0;
extern int          StopLoss       = 0;
extern bool         BreakEven      = true;
extern int          Slippage       = 4;
extern int          Magic          = 6225;

int ticket, CountSellLimit, CountBuyLimit;
double Lot;
double PriceSellLimit, PriceBuyLimit, 
       SLSellLimit, SLBuyLimit, 
       TPSellLimit, TPBuyLimit;
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
{
   
   return(INIT_SUCCEEDED);
}
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{
   
}
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
{
   TrailingOrders();

   if (CountTrades(Magic) == 0)
   {
      PriceSellLimit  = NormalizeDouble(Ask + Indent*Point, Digits);
      PriceBuyLimit   = NormalizeDouble(Bid - Indent*Point, Digits);
      Lot = Lots;
      if (Lots == 0)
      {
         if (StopLoss > 0)  Lot = GetLot(StopLoss);
         if (StopLoss == 0) Lot = LotsOptimized();
      }
      for (int i = 1; i <=CountOrders; i++)
      {
         SLSellLimit = NormalizeDouble(PriceSellLimit + StopLoss*Point, Digits);
         if (StopLoss == 0) SLSellLimit = 0;
         TPSellLimit = NormalizeDouble(PriceSellLimit - TakeProfit*Point, Digits);
         if (TakeProfit == 0) TPSellLimit = 0;
         ticket = OrderSend(Symbol(), OP_SELLLIMIT, Lot, PriceSellLimit, Slippage, SLSellLimit, TPSellLimit, "", Magic);
         PriceSellLimit  = NormalizeDouble(PriceSellLimit + DistanceOrders*Point, Digits);
         
         SLBuyLimit = NormalizeDouble(PriceBuyLimit - StopLoss*Point, Digits);
         if (StopLoss == 0) SLBuyLimit = 0;
         TPBuyLimit = NormalizeDouble(PriceBuyLimit + TakeProfit*Point, Digits);
         if (TakeProfit == 0) TPBuyLimit = 0;
         ticket = OrderSend(Symbol(), OP_BUYLIMIT, Lot, PriceBuyLimit, Slippage, SLBuyLimit, TPBuyLimit, "", Magic);
         PriceBuyLimit  = NormalizeDouble(PriceBuyLimit - DistanceOrders*Point, Digits);
      }
   }
   
   if (CountTrades(Magic) > 0)
   {
      if (CountTrades(Magic, OP_SELLLIMIT) < CountSellLimit)
      {
         PriceSellLimit  = NormalizeDouble(MaxPrice(OP_SELLLIMIT) + DistanceOrders*Point, Digits);
         SLSellLimit = NormalizeDouble(PriceSellLimit + StopLoss*Point, Digits);
         if (StopLoss == 0) SLSellLimit = 0;
         TPSellLimit = NormalizeDouble(PriceSellLimit - TakeProfit*Point, Digits);
         if (TakeProfit == 0) TPSellLimit = 0;
         ticket = OrderSend(Symbol(), OP_SELLLIMIT, Lot, PriceSellLimit, Slippage, SLSellLimit, TPSellLimit, "", Magic);
      }
      if (CountTrades(Magic, OP_BUYLIMIT)  < CountBuyLimit)
      {
         PriceBuyLimit  = NormalizeDouble(MinPrice(OP_BUYLIMIT) - DistanceOrders*Point, Digits);
         SLBuyLimit = NormalizeDouble(PriceBuyLimit - StopLoss*Point, Digits);
         if (StopLoss == 0) SLBuyLimit = 0;
         TPBuyLimit = NormalizeDouble(PriceBuyLimit + TakeProfit*Point, Digits);
         if (TakeProfit == 0) TPBuyLimit = 0;
         ticket = OrderSend(Symbol(), OP_BUYLIMIT, Lot, PriceBuyLimit, Slippage, SLBuyLimit, TPBuyLimit, "", Magic);
      }
   }
   
   CountSellLimit = CountTrades(Magic, OP_SELLLIMIT);
   CountBuyLimit  = CountTrades(Magic, OP_BUYLIMIT);
}
//+------------------------------------------------------------------+
int CountTrades(int magic = -1, int type = -1)
{
   int count = 0;
   
   for (int trade=OrdersTotal()-1; trade>=0; trade--)
   {
      if (OrderSelect(trade, SELECT_BY_POS))
      {
         if (OrderSymbol() == Symbol() && OrderMagicNumber() == magic && (OrderType() == type || type == -1))
            count++;
      }
   }
   return (count);
}
//+------------------------------------------------------------------+
double GetLot(int sl)
{double Free =AccountFreeMargin();
double One_Lot =MarketInfo(Symbol(),MODE_MARGINREQUIRED);
double LotVal =MarketInfo(Symbol(),MODE_TICKVALUE);
double Min_Lot =MarketInfo(Symbol(),MODE_MINLOT);
double Max_Lot =MarketInfo(Symbol(),MODE_MAXLOT);
double Step =MarketInfo(Symbol(),MODE_LOTSTEP);
double lot =MathFloor((Free*Risk/100)/(sl*LotVal)/Step)*Step;
if(lot<Min_Lot) lot=Min_Lot;
if(lot>Max_Lot) lot=Max_Lot;
if(lot*One_Lot>Free) return(0.0);
return(lot);}
//+------------------------------------------------------------------+
double LotsOptimized() 
{return(NormalizeDouble((AccountBalance()/1000)*MarketInfo(Symbol(), MODE_MINLOT)*Risk, 2));}
//+------------------------------------------------------------------+
double MinPrice(int type)
{
   double price = 10000;
   for (int i=OrdersTotal()-1; i>=0; i--)
   {
      if (OrderSelect(i, SELECT_BY_POS))
      {
         if (OrderSymbol() == Symbol() && OrderMagicNumber() == Magic && OrderType() == type)
         {
            if (OrderOpenPrice() < price) price = OrderOpenPrice();
         }
      }
   }
   return(price);
}
//+------------------------------------------------------------------+
double MaxPrice(int type)
{
   double price = 0;
   for (int i=OrdersTotal()-1; i>=0; i--)
   {
      if (OrderSelect(i, SELECT_BY_POS))
      {
         if (OrderSymbol() == Symbol() && OrderMagicNumber() == Magic && OrderType() == type)
         {
            if (OrderOpenPrice() > price) price = OrderOpenPrice();
         }
      }
   }
   return(price);
}
//+------------------------------------------------------------------+
int CountPreviousOrders(double price, int type)
{
   int cnt = 0;

   for (int i =OrdersTotal()-1; i>=0; i--)
   {
      if (OrderSelect(i, SELECT_BY_POS))
      {
         if (OrderSymbol() == Symbol() && OrderMagicNumber() == Magic && OrderType() == type)
         {
            if (type == OP_SELLLIMIT && OrderOpenPrice() < price) cnt++;
            if (type == OP_BUYLIMIT  && OrderOpenPrice() > price) cnt++;
         }
      }
   }
   return(cnt);
}
//+------------------------------------------------------------------+
void TrailingOrders()
{
   double NewPrice, NewSL = 0, NewTP = 0;
   double minprice, maxprice;
   bool   mdf;
   for (int i=0; i < OrdersTotal(); i++)
   {
      if (OrderSelect(i, SELECT_BY_POS))
      {
         if (OrderSymbol() == Symbol() && OrderMagicNumber() == Magic)
         {
            if (OrderType() == OP_SELLLIMIT && CountTrades(Magic, OP_SELL) == 0)
            {
               Print(OrderTicket());
               minprice = NormalizeDouble(OrderOpenPrice() - (DistanceOrders*CountPreviousOrders(OrderOpenPrice(), OP_SELLLIMIT)), Digits);
               if (minprice >= Bid+(TrailingStop+TrailingStep)*Point)
               {
                  NewPrice = NormalizeDouble(Bid+(TrailingStop*Point)+(DistanceOrders*CountPreviousOrders(OrderOpenPrice(), OP_SELLLIMIT)), Digits);
                  if (StopLoss > 0)   NewSL = NormalizeDouble(NewPrice+StopLoss*Point, Digits); else NewSL = 0;
                  if (TakeProfit > 0) NewTP = NormalizeDouble(NewPrice-TakeProfit*Point, Digits); else NewTP = 0;
                  mdf = OrderModify(OrderTicket(), NewPrice, NewSL, NewTP, 0);
               }
            }
            if (OrderType() == OP_BUYLIMIT  && CountTrades(Magic, OP_BUY) == 0)
            {
               maxprice = NormalizeDouble(OrderOpenPrice() + (DistanceOrders*CountPreviousOrders(OrderOpenPrice(), OP_BUYLIMIT)), Digits);
               if (maxprice <= Ask-(TrailingStop+TrailingStep)*Point)
               {
                  NewPrice = NormalizeDouble(Ask-(TrailingStop*Point)-(DistanceOrders*CountPreviousOrders(OrderOpenPrice(), OP_BUYLIMIT)), Digits);
                  if (StopLoss > 0)   NewSL = NormalizeDouble(NewPrice-StopLoss*Point, Digits); else NewSL = 0;
                  if (TakeProfit > 0) NewTP = NormalizeDouble(NewPrice+TakeProfit*Point, Digits); else NewTP = 0;
                  mdf = OrderModify(OrderTicket(), NewPrice, NewSL, NewTP, 0);
               }
            }
         }
      }
   }
}